#!/usr/bin/env python3

import yaml
import networkx as nx
from networkx.drawing.nx_agraph import to_agraph
#import pygraphviz as gv
import os
import itertools

G = nx.OrderedDiGraph()

with open('proofcomplexityzoo.yaml','r') as f:
    data = yaml.load(f)

#print(data)
for ps,meta in data["proofsystems"].items():
    G.add_node(ps,label=meta["name"],shape="egg",style="filled",fillcolor="lightgray",penwidth="0",**meta)

for simulation in data["simulations"]:
    strong=simulation["strong"]
    weak=simulation["weak"]
    paper=simulation.get("paper")
    label=""
    if paper:
        label+="\n[{}]".format(paper)
    if G.has_edge(weak,strong):
        G[weak][strong]["constraint"]=False
        G[weak][strong]["dir"]="both"
        G[weak][strong]["label"]+=label
    else:
        G.add_edge(strong,weak,label=label, style="solid")        

for separation in data["separations"]:
    strong=separation["strong"]
    weak=separation["weak"]
    formula=separation.get("formula")
    paper=separation.get("paper")
    label=formula
    constraint=True
    if "strong" in G.node[weak] and "strong" not in G.node[strong]:
        constraint=False
    if paper:
        label+="\n[{}]".format(paper)
    if G.has_edge(strong,weak):
        G.add_edge(strong,weak, style="dashed",label=label)
    elif G.has_edge(weak,strong):
        G[weak][strong]["constraint"]=False
        G[weak][strong]["dir"]="both"
        G[weak][strong]["label"]+="\n"+label
    else:
        G.add_edge(strong,weak, style="dotted",label=label,**separation)

for cycle in nx.simple_cycles(G):
    nextedgeiterator = itertools.cycle(cycle)
    next(nextedgeiterator)
    for u,v in zip(cycle,nextedgeiterator):
        if G[u][v]["style"] == "dotted":
            G[u][v]["constraint"]=False

H = to_agraph(G)

for i,friends in enumerate(data["friends"]):
    H.add_subgraph(friends,name="cluster_{}".format(i),style="filled",fillcolor="#ffffee",penwidth="0",label="")

for samerank in data["samerank"]:
    H.add_subgraph(samerank,rank="same")

    
H.graph_attr["newrank"]=True
H.graph_attr["overlap"]="scale"

H.graph_attr["label"]="Proof Complexity Zoo - 20180830 - Marc Vinyals"
H.graph_attr["labelloc"]="b"
H.graph_attr["labeljust"]="r"
    
H.write("proofcomplexityzoo.dot")
os.system("dot -T pdf proofcomplexityzoo.dot -o proofcomplexityzoo.pdf")
